/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/*
 * $Id: device_file.h,v 1.7 2002/06/15 15:16:14 mulix Exp $
 */ 

#ifndef DEVICE_FILE_H_
#define DEVICE_FILE_H_

#include <linux/kernel.h>
#include <linux/types.h>  /* size_t */
#include <linux/fs.h>     /* everything... */

/* the device has a request buffer, for copying the requst from user */ 
/* space, a response buffer, for copying the response back to user */ 
/* space, and a semaphore, to make sure only one client gets access */ 
/* to the device at any given time */ 
typedef struct sct_dev_type {
	sct_membuf_t* req; 
	sct_membuf_t* resp; 
	struct semaphore sem;     /* Mutual exclusion */
} sct_dev_t;

/* the major device number for the device file */
extern int sct_major; 

/* the device struct */
extern sct_dev_t sct_dev;

/* the file operations for the device */
extern struct file_operations sct_fops;

/* call these on module_init and module_shutdown */ 
int init_dev_file(sct_dev_t* dev); 
void destroy_dev_file(sct_dev_t* dev); 

#endif /* DEVICE_FILE_H_ */
