/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 guy keren, choo@actcom.co.il             */
/* License: GNU General Public License                              */

/* 
 * $Id: logger_helpers.c,v 1.3 2002/06/15 15:16:14 mulix Exp $
 */ 

#include <linux/kernel.h>
#include <linux/unistd.h>
#include <asm/byteorder.h>

#include "logger_helpers.h"

#include "rule_engine_public.h"


/*
 * Private functions.
 */

/* emit a binary-formated IP address as a dotted-notation string */
int
lhf_inet_ntoa(void* p_in, char* buf, int buflen)
{
    int rc;
    char loc_buf[18];
    char *p;

    p = (char *) p_in;
    sprintf(loc_buf, "%d.%d.%d.%d",
	    (p[0] & 255), (p[1] & 255), (p[2] & 255), (p[3] & 255));

    rc = sct_tracker_stringify_params(buf, buflen, "s", loc_buf);

    return rc;
}

/* emit a network byte-order short in host byte-order. */
int
lhf_ntohs(void* p_s, char* buf, int buflen)
{
    int rc;

    if (p_s) {
        unsigned short s = ntohs(*(unsigned short*)p_s);
        rc = sct_tracker_stringify_params(buf, buflen, "uh", s);
    }
    else
        rc = sct_tracker_stringify_params(buf, buflen, "s", "(err..)");

    return rc;
}

/* emit a network byte-order long in host byte-order. */
int
lhf_ntohl(void* p_l, char* buf, int buflen)
{
    int rc;

    if (p_l) {
        unsigned long l = ntohl(*(unsigned long*)p_l);
        rc = sct_tracker_stringify_params(buf, buflen, "ul", l);
    }
    else
        rc = sct_tracker_stringify_params(buf, buflen, "s", "(err..)");

    return rc;
}


/*
 * Public functions.
 */

/* entry logger helper fucntion. */
extern int logger_helper_mux(char spec_char,
			     char* buf,
			     int buflen,
			     void* p,
			     void* p_data,
			     void* cb_data)
{
    int rc = 0;  /* assume failure. */
    logger_helper_funcs_t lhf = (logger_helper_funcs_t)p_data;

    switch (lhf) {
        case LHF_INET_NTOA:
	    rc = lhf_inet_ntoa(p, buf, buflen);
	    break;
        case LHF_NTOHS:
	    rc = lhf_ntohs(p, buf, buflen);
	    break;
        case LHF_NTOHL:
	    rc = lhf_ntohl(p, buf, buflen);
	    break;
	default:
	    break;
    }

    return rc;
}

