/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/*
 *$Id: sct_compat.h,v 1.6 2002/06/15 15:16:14 mulix Exp $
 */

#ifndef SCT_COMPAT_H_
#define SCT_COMPAT_H_

#include <linux/version.h>

/* ugly hackery to support both 2.2 style get_module_symbol and 2.4 */
#if ( LINUX_VERSION_CODE <= KERNEL_VERSION(2,4,0) )

/* DON'T mess with these - they are required to allow proper expansion of */
/* (possibly) versioned symbols into strings.  -                          */
#define do_turn_into_versioned_string(x) #x                            
#define versioned_string(x) do_turn_into_versioned_string(x)          

#else /* we're running 2.4. good */

#define versioned_string(x) (x)
#undef get_module_symbol 
#define get_module_symbol(x,y) (y)

#endif /* LINUX_VERSION_CODE */

/* mutex interface changed */

#if ( LINUX_VERSION_CODE <= KERNEL_VERSION(2,4,0) )

#ifndef DECLARE_MUTEX
# define DECLARE_MUTEX(sem) struct semaphore (sem) = MUTEX
#endif
#ifndef init_MUTEX
# define init_MUTEX(sem) sema_init((sem) ,1) 
#endif
#ifndef SET_MODULE_OWNER
# define SET_MODULE_OWNER(a)
#endif

#endif /* LINUX_VERSION_CODE */

/* modules debugging support */
/* if debugging is enabled, artifically raise the priority  */
/* of the logging info. elevate KERN_NOTICE to '1' to we get  */
/* a log even if klogd is run with 'klogd -c 3' */
#ifdef SCT_MODULE_DEBUG

/* values from include/linux/kernel.h */

#undef KERN_NOTICE
#define KERN_NOTICE "<1>"        /* KERN_ALERT */

#undef KERN_INFO 
#define KERN_INFO "<5>"          /* KERN_NOTICE */

#undef KERN_DEBUG
#define KERN_DEBUG "<5>"         /* KERN_NOTICE */

#endif /* SCT_MODULE_DEBUG */

/* are the MODULE_XXX symbols known? */
#ifndef MODULE_AUTHOR
#define MODULE_AUTHOR(x)
#endif

#ifndef MODULE_DESCRIPTION
#define MODULE_DESCRIPTION(x)
#endif

#ifndef MODULE_LICENSE
#define MODULE_LICENSE(x)
#endif

#endif /* SCT_COMPAT_H_ */
