/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 guy keren, choo@actcom.co.il	            */
/* License: GNU Lesser General Public License                       */

/*
 *$Id: encode_decode_public.h,v 1.5 2002/06/15 15:16:13 mulix Exp $
 */

#ifndef _ENCODE_DECODE_PUBLIC_H
#define _ENCODE_DECODE_PUBLIC_H

/*
 * Rule engine - primitive types encoding/decoding functions.
 */

/* encode the given unsigned char at the given pos, */
/* as long as the buffer is not too small.          */
int encode_uchar(unsigned char* buf, int buflen, unsigned char** pos, unsigned char c);

/* decode the given unsigned char at the given pos, */
/* as long as we are not past end of buf.           */
int decode_uchar(unsigned char* buf, int buflen, unsigned char** pos, unsigned char* pc);

/* encode the given signed char at the given pos, */
/* as long as the buffer is not too small.          */
int encode_char(unsigned char* buf, int buflen, unsigned char** pos, char c);

/* decode the given signed char at the given pos, */
/* as long as we are not past end of buf.           */
int decode_char(unsigned char* buf, int buflen, unsigned char** pos, char* pc);

/* encode the given int at the given pos,  */
/* as long as the buffer is not too small. */
int encode_int(unsigned char* buf, int buflen, unsigned char** pos, int i);

/* decode the given int at the given pos, */
/* as long as we are not past end of buf. */
int decode_int(unsigned char* buf, int buflen, unsigned char** pos, int* pi);

/* encode the given unsigned int at the given pos, */
/* as long as the buffer is not too small.         */
int encode_uint(unsigned char* buf, int buflen,
	        unsigned char** pos, unsigned int i);

/* decode the given unsigned int at the given pos, */
/* as long as we are not past end of buf.          */
int decode_uint(unsigned char* buf, int buflen,
	        unsigned char** pos, unsigned int* pi);

/* encode the given long at the given pos, */
/* as long as the buffer is not too small. */
int encode_long(unsigned char* buf, int buflen, unsigned char** pos, long l);

/* decode the given long at the given pos, */
/* as long aswe are not past end of buf.   */
int decode_long(unsigned char* buf, int buflen, unsigned char** pos, long* pl);

/* encode the given unsigned long at the given pos, */
/* as long as the buffer is not too small.          */
int encode_ulong(unsigned char* buf, int buflen,
		 unsigned char** pos, unsigned long l);

/* decode the given unsigned long at the given pos, */
/* as long aswe are not past end of buf.            */
int decode_ulong(unsigned char* buf, int buflen,
		 unsigned char** pos, unsigned long* pl);

/* encode the given short at the given pos, */
/* as long as the buffer is not too small.  */
int encode_short(unsigned char* buf, int buflen, unsigned char** pos, short s);

/* decode the given short at the given pos, */
/* as long as we are not past end of buf.   */
int decode_short(unsigned char* buf, int buflen, unsigned char** pos, short* ps);

/* encode the given unsigned short at the given pos, */
/* as long as the buffer is not too small.           */
int encode_ushort(unsigned char* buf, int buflen,
		  unsigned char** pos, unsigned short s);

/* decode the given unsigned short at the given pos, */
/* as long as we are not past end of buf.            */
int decode_ushort(unsigned char* buf, int buflen,
		 unsigned char** pos, unsigned short* ps);

/* encode the given float at the given pos, */
/* as long as the buffer is not too small.  */
int encode_float(unsigned char* buf, int buflen, unsigned char** pos, float f);

/* decode the given float at the given pos, */
/* as long as we are nto past end of buf.   */
int decode_float(unsigned char* buf, int buflen, unsigned char** pos, float* pf);

/* encode the given string at the given pos, */
/* as long as the buffer is not too small.   */
int encode_string(unsigned char* buf, int buflen, unsigned char** pos, const char* str);

/* decode the given string at the given pos, */
/* as long as we are not past end of buf.    */
int decode_string(unsigned char* buf, int buflen, unsigned char** pos, char** pstr);

/* encode the given buffer 'in' at the given pos, */
/* as long as the destination buffer 'out' is not too small. */
int
encode_buffer(unsigned char* out, int buflen, unsigned char** pos, 
	      const void* in, unsigned int size); 

/* decode the given buffer '*pdst' at the given pos, */
/* as long as we are not past end of the buf 'src'.  */
int decode_buffer(unsigned char* src, int buflen, unsigned char** pos,
		  void** pdst, unsigned int* psize); 

#endif /* _ENCODE_DECODE_PUBLIC_H */
