/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 guy keren, choo@actcom.co.il	            */
/* License: GNU Lesser General Public License                       */

/*
 *$Id: filter_vars_public.h,v 1.11 2002/06/15 15:16:13 mulix Exp $
 */
#ifndef _KERNEL_VARS_PUBLIC_H
#define _KERNEL_VARS_PUBLIC_H

#include "rule_engine_public.h"

/*
 * Kernal variables manipulation code.
 */

/* empty variable values array. */
void reset_all_vars(void);

/* callback function type */
typedef sct_filter_node_t* (*callback_func)(sct_var_type_t, void*);
typedef sct_filter_node_t* (*callback_indexed_func)(sct_var_type_t, int, 
						    sct_data_type_t, void*);
typedef sct_filter_node_t* (*callback_indexed_field_func)(sct_var_type_t, int, 
							  int, sct_data_type_t, 
							  void*, int);

/* store a given variable's value. */
void set_var(sct_var_type_t type, sct_filter_node_t* val);
void set_var_callback(sct_var_type_t type, callback_func func, void* data);
void set_persistent_callback(sct_var_type_t type, callback_func func, 
			     void* data);
void set_var_field(sct_var_type_t type, int field_index, 
		   sct_filter_node_t* val);
void set_var_indexed(sct_var_type_t type, int var_index, 
		     sct_filter_node_t* val);
void set_var_indexed_callback(sct_var_type_t type, int var_index, 
			      callback_indexed_func func, 
			      sct_data_type_t dtype, void* data);
void set_var_indexed_callback2(sct_var_type_t type, int var_index, 
			       callback_indexed_func func, 
			       sct_data_type_t dtype, void* data1, 
			       void* data2);
void set_var_indexed_field_callback(sct_var_type_t type, int var_index, 
				    callback_indexed_field_func func, 
				    sct_data_type_t dtype, 
				    void* data);

/* evaluate the value of the given parameter, assuming it was previously */
/* set with set_var/set_var_index.                                       */
sct_filter_node_t* get_var(sct_var_type_t type);
sct_filter_node_t* get_var_indexed(sct_var_type_t type, int var_index);
sct_filter_node_t* get_var_indexed_field(sct_var_type_t type,
				     int var_index,
				     int field_num,
				     int struct_type);

/* setting of attributes for variables.                             */
/* currently all attributes have numeric names, and numeric values. */

/* set a variable's attribute. gets reset when calling 'rest_all_vars()'. */
void set_var_attr(sct_var_type_t type, sct_var_attr_name_t name, int val);
void set_var_indexed_attr(sct_var_type_t type, int var_index, 
			  sct_var_attr_name_t name, int val);

/* get a variable's attribute. returns '1' if its defined, '0' if not. */
/* value itself stores in '*p_val'.                                    */
int get_var_attr(sct_var_type_t type, sct_var_attr_name_t name, int* p_val);
int get_var_indexed_attr(sct_var_type_t type, int var_index, 
			 sct_var_attr_name_t name, int* p_val);

/* buffer parameters (which have a void* and a count part) */
/* are returned as this struct */ 
struct sct_aggregate_data2 {
    void* v1; 
    void* v2; 
}; 

#endif /* _KERNEL_VARS_PUBLIC_H */
