/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 guy keren, choo@actcom.co.il	            */
/* License: GNU General Public License                              */

/*
 *$Id: main.c,v 1.25 2002/06/15 15:16:13 mulix Exp $
 */

#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>

#include "rule_engine_public.h"
#include "filter_vars_public.h"

/* print the filter, evaluate it, print the results. */
sct_exp_error_t
eval_and_sct_print_filter_and_result(sct_filter_node_t* f, sct_filter_node_t* result)
{
    sct_exp_error_t rc;

    printf("\n");
    printf("Evaluating filter:\n");
    sct_print_filter_node(f);
    rc = sct_eval_filter_node(f, result);
    printf("sct_eval_filter_node returned '%d'\n", rc);
    if (rc == ERR_NONE) {
	printf("result:\n");
	sct_print_filter_node(result);
    }
    printf("\n");

    return rc;
}

void
test_negation(void)
{
    sct_filter_node_t* result = sct_make_bool_val(TF_FALSE);
    
    printf("\n\n\n-------------------\n");
    printf("Testing bit negation\n");
    {
	sct_filter_node_t* val = sct_make_ulong_val(1);
	sct_filter_node_t* op = sct_make_bit_not(val);

	eval_and_sct_print_filter_and_result(op, result);

	sct_free_filter_node(op);
    }
    printf("Testing logical negation\n");
    {
	sct_filter_node_t* val = sct_make_long_val(0);
	sct_filter_node_t* op = sct_make_logical_not(val);

	eval_and_sct_print_filter_and_result(op, result);
	
	sct_free_filter_node(op);
    }
    sct_free_filter_node(result);
}
    
/* kernel variables assignment testing. */
void
test_variables(void)
{
    sct_filter_node_t* result = sct_make_bool_val(TF_FALSE);
    sct_exp_error_t rc;

    printf("\n\n\n-------------------\n");
    printf("Testing variables\n");
    {
	sct_filter_node_t* v_sys_id = sct_make_int_val(10);
	sct_filter_node_t* v_param_0 = sct_make_string_val("/etc/passwd");
	sct_filter_node_t* f0 = sct_make_var_val(VT_SYSCALL_ID, DT_INT);
	sct_filter_node_t* f1 =
	    sct_make_var_indexed_val(VT_PARAM, 0, DT_STRING);

	reset_all_vars();
	set_var(VT_SYSCALL_ID, v_sys_id);
	set_var_indexed(VT_PARAM, 0, v_param_0);

	rc = sct_eval_filter_node(f0, result);
	printf("syscall_id (%d):\n", rc);
	sct_print_filter_node(result);

	rc = sct_eval_filter_node(f1, result);
	printf("param 0 (%d):\n", rc);
	sct_print_filter_node(result);
	
	sct_free_filter_node(f1);
	sct_free_filter_node(f0);
    }
    {
	sct_exp_error_t rc;
	sct_filter_node_t* v_param_0 = sct_make_filter_vector(2);
	sct_filter_node_t* f0 = sct_make_string_val("/bli/bla");
	sct_filter_node_t* f1 = sct_make_int_val(7);
	sct_filter_node_t* filter_0_0 =
	    sct_make_var_indexed_field_val(VT_PARAM, 0, 0, DT_STRING, -1);
	sct_filter_node_t* filter_0_1 =
	    sct_make_var_indexed_field_val(VT_PARAM, 0, 1, DT_STRING, -1);
	sct_insert_filter_node(v_param_0, 0, f0);
	sct_insert_filter_node(v_param_0, 1, f1);

	reset_all_vars();
	set_var_indexed(VT_PARAM, 0, v_param_0);

	rc = sct_eval_filter_node(filter_0_0, result);
	printf("param 0.0 (%d):\n", rc);
	sct_print_filter_node(result);

	rc = sct_eval_filter_node(filter_0_1, result);
	printf("param 0.1 (%d):\n", rc);
	sct_print_filter_node(result);

	sct_free_filter_node(filter_0_0);
	sct_free_filter_node(filter_0_1);
    }

    sct_free_filter_node(result);
}

/* check short circuiting */
void
test_short_circuiting(void)
{
    sct_filter_node_t* result = sct_make_bool_val(TF_FALSE);
    sct_exp_error_t rc;

    printf("\n\n\n-------------------\n");
    printf("Testing short-circuiting\n");

    {
	sct_filter_node_t* f0 = sct_make_int_val(0);
	sct_filter_node_t* f1 = sct_make_int_val(1);
	sct_filter_node_t* f_and = sct_make_logical_and(f0, f1);

	printf("short circuiting support test: and\n");
	rc = eval_and_sct_print_filter_and_result(f_and, result);
 	
	sct_free_filter_node(f_and);
    }
    {
	sct_filter_node_t* f0 = sct_make_int_val(1);
	sct_filter_node_t* f1 = sct_make_int_val(0);
	sct_filter_node_t* f_and = sct_make_logical_and(f0, f1);

	printf("short circuiting support test: and\n");
	rc = eval_and_sct_print_filter_and_result(f_and, result);
 	
	sct_free_filter_node(f_and);
    }
    {
	sct_filter_node_t* f0 = sct_make_int_val(1);
	sct_filter_node_t* f1 = sct_make_int_val(0);
	sct_filter_node_t* f_or = sct_make_logical_or(f0, f1);

	printf("short circuiting support test: or\n");
	rc = eval_and_sct_print_filter_and_result(f_or, result);
 	
	sct_free_filter_node(f_or);
    }
    {
	sct_filter_node_t* f0 = sct_make_int_val(0);
	sct_filter_node_t* f1 = sct_make_int_val(1);
	sct_filter_node_t* f_or = sct_make_logical_or(f0, f1);

	printf("short circuiting support test: or\n");
	rc = eval_and_sct_print_filter_and_result(f_or, result);
 	
	sct_free_filter_node(f_or);
    }

    sct_free_filter_node(result);
}	

/* filter evaluation testing */
void
test_filter_eval_bit(void)
{
    sct_filter_node_t* result = sct_make_bool_val(TF_FALSE);
    sct_exp_error_t rc;

    {
	sct_filter_node_t* f0 = sct_make_long_val(120L);
	sct_filter_node_t* f1 = sct_make_long_val(30L);
	sct_filter_node_t* f2 = sct_make_bit_and(f0, f1);

	rc = eval_and_sct_print_filter_and_result(f2, result);

	sct_free_filter_node(f2);
    }
    {
	sct_filter_node_t* f0 = sct_make_long_val(120L);
	sct_filter_node_t* f1 = sct_make_long_val(30L);
	sct_filter_node_t* f2 = sct_make_bit_or(f0, f1);

	rc = eval_and_sct_print_filter_and_result(f2, result);

	sct_free_filter_node(f2);
    }
    {
	sct_filter_node_t* f0 = sct_make_long_val(120L);
	sct_filter_node_t* f1 = sct_make_long_val(30L);
	sct_filter_node_t* f2 = sct_make_bit_xor(f0, f1);

	rc = eval_and_sct_print_filter_and_result(f2, result);

	sct_free_filter_node(f2);
    }

    sct_free_filter_node(result);
}

/* test unsigned data types. */
void
test_unsigned_types(void)
{
    sct_filter_node_t* result = sct_make_bool_val(TF_FALSE);
    sct_exp_error_t rc;

    printf("\n\n\n");
    printf("Testing unsigned data types\n\n");

    {
	sct_filter_node_t* f0 = sct_make_ulong_val(120L);
	sct_filter_node_t* f1 = sct_make_ulong_val(30L);
	sct_filter_node_t* f2 = sct_make_math_plus(f0, f1);

	rc = eval_and_sct_print_filter_and_result(f2, result);

	sct_free_filter_node(f2);
    }
    {
	sct_filter_node_t* f0 = sct_make_ushort_val(120);
	sct_filter_node_t* f1 = sct_make_ushort_val(30);
	sct_filter_node_t* f2 = sct_make_math_plus(f0, f1);

	rc = eval_and_sct_print_filter_and_result(f2, result);

	sct_free_filter_node(f2);
    }
    {
	sct_filter_node_t* f0 = sct_make_uint_val(120);
	sct_filter_node_t* f1 = sct_make_uint_val(30);
	sct_filter_node_t* f2 = sct_make_math_plus(f0, f1);

	rc = eval_and_sct_print_filter_and_result(f2, result);

	sct_free_filter_node(f2);
    }
    {
	sct_filter_node_t* f0 = sct_make_ushort_val(65000);
	sct_filter_node_t* f1 = sct_make_uint_val(100820);
	sct_filter_node_t* f2 = sct_make_math_plus(f0, f1);

	rc = eval_and_sct_print_filter_and_result(f2, result);

	sct_free_filter_node(f2);
    }
    {
	sct_filter_node_t* f0 = sct_make_short_val(65000);
	sct_filter_node_t* f1 = sct_make_uint_val(100820);
	sct_filter_node_t* f2 = sct_make_math_plus(f0, f1);

	rc = eval_and_sct_print_filter_and_result(f2, result);

	sct_free_filter_node(f2);
    }
    {
	sct_filter_node_t* f0 = sct_make_ushort_val(64003);
	sct_filter_node_t* f1 = sct_make_short_val(5);
	sct_filter_node_t* f2 = sct_make_bit_left_shift(f0, f1);

	rc = eval_and_sct_print_filter_and_result(f2, result);

	sct_free_filter_node(f2);
    }
    printf("\n\n\n");
    sct_free_filter_node(result);
}

/* test filter node vectors. */
void
test_filter_node_vectors(void)
{
    printf("\n\n\n");
    printf("Testing filter node vectors\n\n");

    {
	int rc;
	sct_filter_node_t* f0 = sct_make_ulong_val(120L);
	sct_filter_node_t* f1 = sct_make_ulong_val(30L);
	sct_filter_node_t* f2 = sct_make_math_plus(f0, f1);
	sct_filter_node_t* f3 = sct_make_bool_val(0);
	sct_filter_node_t* f4 = sct_make_int_val(10);
	sct_filter_node_t* f5 = sct_make_int_val(20);
	sct_filter_node_t* f6 = sct_make_equal(f4, f5);
	sct_filter_node_t* vec = sct_make_filter_vector(5);

	rc = sct_insert_filter_node(vec, 0, f2);
	rc = sct_insert_filter_node(vec, 1, f3);
	rc = sct_insert_filter_node(vec, 2, f6);

	sct_print_filter_node(vec);

	sct_free_filter_node(vec);
    }
    printf("\n\n\n");
}

/* test in match. */
void
test_in_match(void)
{
    printf("\n\n\n");
    printf("Testing in match\n\n");

    {
	int rc;
	sct_filter_node_t* result = sct_make_bool_val(TF_FALSE);
	sct_filter_node_t* f1 = sct_make_int_val(10);
	sct_filter_node_t* f2 = sct_make_int_val(20);
	sct_filter_node_t* f3 = sct_make_int_val(20);
	sct_filter_node_t* f4 = sct_make_int_val(30);
	sct_filter_node_t* f5 = sct_make_int_val(40);
	sct_filter_node_t* f6 = sct_make_equal(f3, f4);
	sct_filter_node_t* vec2 = sct_make_filter_vector(2);
	sct_filter_node_t* f7 = sct_make_equal(f5, vec2);
	sct_filter_node_t* vec = sct_make_filter_vector(5);
	sct_filter_node_t* f10 = sct_make_int_val(20);
	sct_filter_node_t* in = sct_make_in_match(f10, vec);

	rc = sct_insert_filter_node(vec, 0, f1);
	rc = sct_insert_filter_node(vec, 1, f2);
	rc = sct_insert_filter_node(vec, 2, f6);
	rc = sct_insert_filter_node(vec, 3, f7);

	sct_print_filter_node(vec);

	rc = sct_eval_filter_node(in, result);
	sct_print_filter_node(result);
	sct_free_filter_node(in);
	sct_free_filter_node(result);
    }
    printf("\n\n\n");
}

void 
test_pattern_match(void)
{
    printf("\n\n\n");
    printf("Testing pattern match\n\n");
    {
	    int rc;
	    sct_filter_node_t* result = sct_make_bool_val(TF_FALSE);
	    sct_filter_node_t* f1 = sct_make_string_val(NULL);
	    sct_filter_node_t* f2 = sct_make_string_val("world");

	    sct_filter_node_t* f3 = sct_make_pattern_match(f1, f2);
	    
	    sct_print_filter_node(f3);

	    rc = sct_eval_filter_node(f3, result);
	    sct_print_filter_node(result);

	    sct_free_filter_node(f3);
	    sct_free_filter_node(result);
    }
    printf("\n\n\n");
}

sct_filter_node_t* callback_syscall_id (sct_var_type_t type, void *data) {
	sct_filter_node_t* f0 = sct_make_string_val((char*)data);
	printf("syscall_id callback function is called.\n");
	return f0;
}

sct_filter_node_t* callback_param (sct_var_type_t type, int param_index, sct_data_type_t dtype, void *data) {
	sct_filter_node_t* f0 = sct_make_string_val((char*)data);
	printf("callback_param() is me.\n");
	return f0;
}

void
test_logging(void)
{
    int dummy;
    sct_action_t* a1 = NULL;
    sct_filter_node_t* proc_uid = sct_make_long_val((long)0);
    sct_filter_node_t* proc_euid = sct_make_long_val((long)1);
    sct_filter_node_t* proc_suid = sct_make_long_val((long)2);
    sct_filter_node_t* proc_gid = sct_make_long_val((long)100);
    sct_filter_node_t* proc_egid = sct_make_long_val((long)101);
    sct_filter_node_t* proc_sgid = sct_make_long_val((long)102);
    sct_filter_node_t* proc_pid = sct_make_long_val((long)980);
    sct_filter_node_t* proc_comm = sct_make_string_val("blabla");
    char *param0 = "param 0 is a cow";

    printf("\n\n\n");
    printf("Testing logging\n\n");

    reset_all_vars();
    set_var_indexed_callback(VT_PARAM, 0, callback_param, DT_STRING, param0);
    set_var(VT_PROC_UID, proc_uid);
    set_var(VT_PROC_EUID, proc_euid);
    set_var(VT_PROC_SUID, proc_suid);
    set_var(VT_PROC_GID, proc_gid);
    set_var(VT_PROC_EGID, proc_egid);
    set_var(VT_PROC_SGID, proc_sgid);
    set_var(VT_PROC_PID, proc_pid);
    set_var(VT_PROC_COMM, proc_comm);

    a1 = sct_make_action_log(4, "invoked: %pid[%comm]: %sid_%sname(%params)\n");
    
    sct_tracker_perform_action(a1, 1, 10, &dummy, "unlink", "P", 0);
    sct_tracker_perform_action(a1, 2, 5, &dummy, "open", "s P i i", "passwd", 0, 32768, 15);
    sct_tracker_perform_action(a1, 3, 10, &dummy, "unlink", "s S uh", "passwd", a1, 64000);

    printf("\n\n\n");
    
    sct_free_action(a1);
    reset_all_vars();
}

void
test_var_callback(void)
{
    sct_filter_node_t* result = sct_make_bool_val(TF_FALSE);
    sct_exp_error_t rc;

    printf("\n\n\n-------------------\n");
    printf("Testing variables\n");
    {
	sct_filter_node_t* f0 = sct_make_var_val(VT_SYSCALL_ID, DT_INT);
	sct_filter_node_t* f1 =
	    sct_make_var_indexed_val(VT_PARAM, 0, DT_STRING);
	char *string = "callback var test";
	char *param0 = "param 0 is a cow";
	sct_filter_node_t* f2 = sct_make_string_val(string);
	sct_filter_node_t* f3 = sct_make_equal(f0, f2);

	reset_all_vars();
	set_persistent_callback(VT_SYSCALL_ID, callback_syscall_id, string);
	set_var_indexed_callback(VT_PARAM, 0, callback_param, DT_STRING, param0);
	
	rc = sct_eval_filter_node(f0, result);
	printf("syscall_id (%d):\n", rc);
	sct_print_filter_node(result);

	rc = sct_eval_filter_node(f1, result);
	printf("param[0] (%d):\n", rc);
	sct_print_filter_node(result);

	rc = sct_eval_filter_node(f0, result);
	printf("syscall_id (%d):\n", rc);
	sct_print_filter_node(result);

	rc = sct_eval_filter_node(f1, result);
	printf("param[0] (%d):\n", rc);
	sct_print_filter_node(result);

	rc = sct_eval_filter_node(f3, result);
	printf("equal (%d):\n", rc);
	sct_print_filter_node(result);

	reset_all_vars();
	
	rc = sct_eval_filter_node(f0, result);
	printf("syscall_id (%d):\n", rc);
	sct_print_filter_node(result);

	reset_all_vars();
	
	sct_free_filter_node(f1);
	sct_free_filter_node(f3);
    }
    sct_free_filter_node(result);
}

int
main(int argc, char* argv[])
{
    struct timeval t1, t2;
    sct_filter_node_t* result = sct_make_bool_val(TF_FALSE);
    sct_exp_error_t rc;
    sct_filter_node_t* f0 = sct_make_string_val("pw");
    sct_filter_node_t* vec = sct_make_filter_vector(2);
    sct_filter_node_t* v0 = sct_make_string_val("passwd");
    sct_filter_node_t* v1 = sct_make_string_val("/etc/nsswitch");
    sct_filter_node_t* f2;
    sct_filter_node_t* f3;
    sct_filter_node_t* f4;
    sct_rule_list_t* rules;

    sct_insert_filter_node(vec, 0, v0);
    sct_insert_filter_node(vec, 1, v1);
    f2 = sct_make_in_match(f0, vec);
    f3 = sct_make_int_val(15);
    f4 = sct_make_logical_and(f2, f3);

    printf("Printing filter node.\n");
    sct_print_filter_node(f4);


    gettimeofday(&t1, NULL);
    rc = sct_eval_filter_node(f4, result);
    gettimeofday(&t2, NULL);

    printf("   time before eval: %ld, %ld\n", t1.tv_sec, t1.tv_usec);
    printf("   time after  eval: %ld, %ld\n", t2.tv_sec, t2.tv_usec);
    printf("Expression result (rc = %d):\n", rc);
    sct_print_filter_node(result);

    /* filter serialization/deserializatin testing. */
    {
	unsigned char buf[1000];
	int len = sct_serialize_filter_node(f4, buf, 1000);
	printf("serialized 'f4' into '%d' bytes\n", len);

	if (len > 0) {
	    sct_filter_node_t* f_deser =
		sct_deserialize_filter_node(buf, len);
	    if (f_deser) {
		printf("'f4' deserialized:\n");
		sct_print_filter_node(f_deser);
		sct_free_filter_node(f_deser);
	    }
	    else {
		printf("Deserialization of 'f4' failed.\n");
	    }
	}
    }
	
    /* tracking rules serialization/de-serialization testing. */
    {
	unsigned char buf[1000];
	sct_action_t* a1 = sct_make_action_log(4, NULL);
	int len = sct_serialize_tracking_rule(1, RW_BEFORE, 10, a1, f4, buf, 1000);
	printf("serialized rule 1 into '%d' bytes\n", len);

	if (len > 0) {
	    sct_action_t* a_deser;
	    sct_filter_node_t* f_deser;
	    int rule_id;
	    sct_rule_when_t when;
	    int syscall_id;
	    int rc = sct_deserialize_tracking_rule(buf,
						   len,
						   &rule_id,
						   &when,
						   &syscall_id,
						   &a_deser,
						   &f_deser);
	    if (rc > 0) {
		printf("rule 1 deserialized:\n");
		printf(" rule_id - %d\n", rule_id);
		printf(" when - %d (%s)\n", when, when ? "after" : "before");
		printf(" syscall_id - %d\n", syscall_id);
		sct_print_action_node(a_deser);
		sct_print_filter_node(f_deser);
		sct_free_action(a_deser);
		sct_free_filter_node(f_deser);
	    }
	    else {
		printf("Deserialization of rule 1 failed.\n");
	    }
	}
	
	sct_free_action(a1);
    }

    /* tracking rules add/delete testing. */
    {
	sct_action_t* a1 = sct_make_action_log(4, NULL);
	int rc;
		
	printf("creating tracking rules list\n");
	rules = sct_create_tracking_rule_list();
	printf("adding tracking rules\n");
	rc = sct_add_tracking_rule(rules, 6, f4, a1);

	sct_print_tracking_rules(rules);
		
	printf("deleting all tracking rules\n");
	sct_del_all_tracking_rules_in_list(rules);
	sct_print_tracking_rules(rules);
	sct_delete_tracking_rule_list(rules);
    }

    test_variables();

    test_short_circuiting();

    test_filter_eval_bit();

    test_unsigned_types();
	
    test_filter_node_vectors();

    test_negation();

    test_in_match();

    test_pattern_match(); 

    test_var_callback();

    test_logging();

    sct_free_filter_node(result);

    return 0;
}
