/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 guy keren, choo@actcom.co.il	            */
/* License: GNU General Public License                              */

/* 
 * $Id: rule_engine_test.c,v 1.3 2002/06/15 15:16:14 mulix Exp $
 */

#include <assert.h>
#include <stdio.h>

#include "rule_engine_public.h"
#include "rule_engine_private.h"
#include "membuf.h"

#include "../tests/tester.h"

int test_tracking_rule_list(void)
{
	int rc; 
	int rid = 0; 
	sct_rule_list_t* list; 
	sct_membuf_t* mbuf; 

	(void)test_tracking_rule_list(); 

	/* create a tracking rule list */ 
	list = sct_create_tracking_rule_list(); 
	if (!list)
		die("creating an empty tracking rule list failed"); 

	/* add a few rules to the list */ 
	while (++rid < 10){
		sct_filter_node_t* f = sct_make_int_val(rid * 3);
		sct_action_t* a = sct_make_action_log(8, "bla bla"); 
		rc = sct_add_tracking_rule(list, rid, f, a); 
		if (rc < 0)
			die("failed to add a tracking rule to the list"); 
	}

	/* create a membuf */ 
	mbuf = sct_membuf_create(1000); 
	if (!mbuf)
		die("couldn't create a membuf"); 

	/* serialize the list */ 
	rc = sct_serialize_tracking_rule_list(list, mbuf); 
	if (rc <= 0)
		die("serializing the list failed"); 
	
	/* delete the list */ 
	sct_delete_tracking_rule_list(list); 

	rc = sct_deserialize_tracking_rule_list(mbuf, &list); 
	if (rc <= 0) 
		die("failed to deserialize list"); 

	/* print the list */ 
	sct_print_tracking_rules(list); 
	
	sct_membuf_destroy(mbuf); 
	sct_delete_tracking_rule_list(list); 
	return 1; 
}

int test_empty_tracking_rule_list(void)
{
	int rc; 
	sct_membuf_t* mbuf; 

	/* create a tracking rule list */ 
	sct_rule_list_t* list = sct_create_tracking_rule_list(); 
	if (!list)
		die("creating an empty tracking rule list failed"); 

	/* create a membuf */ 
	mbuf = sct_membuf_create(1000); 
	if (!mbuf)
		die("couldn't create a membuf"); 

	/* serialize the list */ 
	rc = sct_serialize_tracking_rule_list(list, mbuf); 
	if (rc <= 0)
		die("serializing the list failed"); 
	
	/* delete the list */ 
	sct_delete_tracking_rule_list(list); 

	rc = sct_deserialize_tracking_rule_list(mbuf, &list); 
	if (rc <= 0) 
		die("failed to deserialize list"); 
	
	sct_membuf_delete(mbuf); 
	sct_delete_tracking_rule_list(list); 
	return 1; 
}
