/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 guy keren, choo@actcom.co.il	            */
/* License: GNU General Public License                              */

/*
 * $Id: tracker_ctrl.c,v 1.8 2002/06/15 15:16:14 mulix Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <linux/unistd.h>
#include <linux/sysctl.h>

#include "rule_engine_public.h"

_syscall1(int, _sysctl, struct __sysctl_args *, args);

void
usage(char* progname)
{
    fprintf(stderr, "usage: %s r1|r2|r3|r4|r5|r6|r7|r8 [syscall_id]\n", progname);
    fprintf(stderr, "       %s u1|u2|u3|u4|u5|u6|u7|u8 [syscall_id]\n", progname);
    fprintf(stderr, "       %s dall syscall_id\n", progname);
    fprintf(stderr,
	    "example: to add rule 1 for the "
	    "open system call, run: %s r1 5\n",
	    progname);
    fprintf(stderr,
	    "example: to delete rule 1 for the "
	    "open system call, run: %s u1 5\n",
	    progname);
    fprintf(stderr,
	    "example: to delete all rules for the "
	    "open system call, run: %s dall 5\n",
	    progname);
    exit(1);
}

int
main(int argc, char* argv[])
{
    sct_filter_node_t* f = NULL;
    sct_action_t* a = NULL;
    unsigned char buf[1000];
    int serial_len;
    int rule_id = 1;
    int syscall_id = 10;
    int do_reg = 1;
    int do_del_all = 0;

    if (argc < 2 || argc > 3) {
	usage(argv[0]);
    }

    if (strcmp(argv[1],"dall") == 0) {
	if (argc != 3)
	    usage(argv[0]);
	do_del_all = 1;
    }
    else if (strcmp(argv[1],"u1") == 0) {
	f = NULL;
	a = NULL;
	do_reg = 0;
	rule_id = 1;
    }
    else if (strcmp(argv[1],"u2") == 0) {
	f = NULL;
	a = NULL;
	do_reg = 0;
	rule_id = 2;
    }
    else if (strcmp(argv[1],"u3") == 0) {
	f = NULL;
	a = NULL;
	do_reg = 0;
	rule_id = 3;
	syscall_id = 12;
    }
    else if (strcmp(argv[1],"u4") == 0) {
	f = NULL;
	a = NULL;
	do_reg = 0;
	rule_id = 4;
	syscall_id = 15;
    }
    else if (strcmp(argv[1],"u5") == 0) {
	f = NULL;
	a = NULL;
	do_reg = 0;
	rule_id = 5;
	syscall_id = 33;
    }
    else if (strcmp(argv[1],"u6") == 0) {
	f = NULL;
	a = NULL;
	do_reg = 0;
	rule_id = 6;
	syscall_id = 9;
    }
    else if (strcmp(argv[1],"u7") == 0) {
	f = NULL;
	a = NULL;
	do_reg = 0;
	rule_id = 7;
	syscall_id = 10;
    }
    else if (strcmp(argv[1],"u8") == 0) {
	f = NULL;
	a = NULL;
	do_reg = 0;
	rule_id = 8;
	syscall_id = 79;
    }
    else if (strcmp(argv[1],"r8") == 0) {
#if 1
	sct_filter_node_t* f0 =
	    sct_make_var_indexed_field_val(VT_PARAM, 1, 1, DT_INT, -1);
	sct_filter_node_t* f1 = sct_make_int_val(0);
	sct_filter_node_t* f2 = sct_make_equal(f0, f1);
#else
	sct_filter_node_t* f2 = sct_make_bool_val(1);
#endif
	sct_action_t* a1 = sct_make_action_log(4, NULL);

	a = a1;
	f = f2;
	do_reg = 1;
	rule_id = 8;
	syscall_id = 79;
    }
    else if (strcmp(argv[1],"r7") == 0) {
	sct_filter_node_t* f0 = sct_make_var_indexed_val(VT_PARAM, 0, DT_STRING);
	sct_filter_node_t* f1 = sct_make_string_val("hash");
	sct_filter_node_t* f2 = sct_make_pattern_match(f0, f1);
	sct_filter_node_t* f3 = sct_make_var_val(VT_PROC_COMM, DT_STRING);
	sct_filter_node_t* f4 = sct_make_string_val("rm");
	sct_filter_node_t* f5 = sct_make_pattern_match(f3, f4);
	sct_filter_node_t* f6 = sct_make_logical_and(f2, f5);
	sct_action_t* a1 = sct_make_action_log(4, NULL);

	a = a1;
	f = f6;
	do_reg = 1;
	rule_id = 7;
	syscall_id = 10;
    }
    else if (strcmp(argv[1],"r6") == 0) {
	sct_filter_node_t* f0 = sct_make_var_indexed_val(VT_PARAM, 0, DT_STRING);
	sct_filter_node_t* f1 = sct_make_string_val("oldfile");
	sct_filter_node_t* f2 = sct_make_pattern_match(f0, f1);
	sct_filter_node_t* f3 = sct_make_var_indexed_val(VT_PARAM, 1, DT_STRING);
	sct_filter_node_t* f4 = sct_make_string_val("newfile");
	sct_filter_node_t* f5 = sct_make_pattern_match(f3, f4);
	sct_filter_node_t* f6 = sct_make_logical_and(f2, f5);
	sct_action_t* a1 = sct_make_action_log(4, NULL);

	a = a1;
	f = f6;
	do_reg = 1;
	rule_id = 6;
	syscall_id = 9;
    }
    else if (strcmp(argv[1],"r5") == 0) {
	sct_filter_node_t* f0 = sct_make_var_indexed_val(VT_PARAM, 0, DT_STRING);
	sct_filter_node_t* f1 = sct_make_string_val("hash");
	sct_filter_node_t* f2 = sct_make_pattern_match(f0, f1);
	sct_action_t* a1 = sct_make_action_log(4, NULL);

	a = a1;
	f = f2;
	do_reg = 1;
	rule_id = 5;
	syscall_id = 33;
    }
    else if (strcmp(argv[1],"r4") == 0) {
	sct_filter_node_t* f0 = sct_make_var_indexed_val(VT_PARAM, 0, DT_STRING);
	sct_filter_node_t* f1 = sct_make_string_val("hash");
	sct_filter_node_t* f2 = sct_make_pattern_match(f0, f1);
	sct_filter_node_t* f3 = sct_make_var_indexed_val(VT_PARAM, 1, DT_LONG);
	sct_filter_node_t* f4 = sct_make_long_val(0700);
	sct_filter_node_t* f5 = sct_make_equal(f3, f4);
	sct_filter_node_t* f6 = sct_make_logical_and(f2, f5);
	sct_action_t* a1 = sct_make_action_log(4, NULL);

	a = a1;
	f = f6;
	do_reg = 1;
	rule_id = 4;
	syscall_id = 15;
    }
    else if (strcmp(argv[1],"r3") == 0) {
	sct_filter_node_t* f0 = sct_make_var_indexed_val(VT_PARAM, 0, DT_STRING);
	sct_filter_node_t* f1 = sct_make_string_val("hash");
	sct_filter_node_t* f2 = sct_make_pattern_match(f0, f1);
	sct_action_t* a1 = sct_make_action_log(4, NULL);

	a = a1;
	f = f2;
	do_reg = 1;
	rule_id = 3;
	syscall_id = 12;
    }
    else if (strcmp(argv[1],"r2") == 0) {
	sct_filter_node_t* f0 = sct_make_var_indexed_val(VT_PARAM, 0, DT_STRING);
	sct_filter_node_t* f1 = sct_make_string_val("hash");
	sct_filter_node_t* f2 = sct_make_pattern_match(f0, f1);
	sct_action_t* a1 = sct_make_action_log(4, NULL);

	a = a1;
	f = f2;
	do_reg = 1;
	rule_id = 2;
    }
    else if (strcmp(argv[1],"r1") == 0) {
	sct_filter_node_t* f0 = sct_make_var_indexed_val(VT_PARAM, 0, DT_STRING);
	sct_filter_node_t* f1 = sct_make_string_val("passwd");
	sct_filter_node_t* f2 = sct_make_pattern_match(f0, f1);
	sct_action_t* a1 = sct_make_action_log(4, NULL);
		
	a = a1;
	f = f2;
	do_reg = 1;
	rule_id = 1;
    }

    if (argc == 3) {
	syscall_id = atoi(argv[2]);
    }

    serial_len = sct_serialize_tracking_rule(rule_id, RW_AFTER,
					 syscall_id, a, f, buf, 900);
    printf("serialized rule into '%d' bytes\n", serial_len);
    if (serial_len > 0)
    {
	int sysctl_add_rule[] = {8, 1};
	int sysctl_del_rule[] = {8, 2};
	int sysctl_del_all_rules[] = {8, 5};
	int* sysctl_action = sysctl_add_rule;

	if (do_del_all) {
	    sysctl_action = sysctl_del_all_rules;
	}
	else {
	    if (do_reg) {
		sysctl_action = sysctl_add_rule;
	    }
	    else {
		sysctl_action = sysctl_del_rule;
	    }
	}

	{
	    struct __sysctl_args args =
		{ sysctl_action, 2, NULL, 0, buf, serial_len};
	    int rc = _sysctl(&args);

	    if (rc == -1) {
		perror("sysctl");
	    }
	    else {
		printf("_sysctl returned '%d'\n", rc);
	    }
	}
    }

    return 0;

}
