/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/* 
 * $Id: sct_rules_private.h,v 1.10 2002/06/15 15:16:14 mulix Exp $
 */

#ifndef SCT_RULES_PRIVATE_H_
#define SCT_RULES_PRIVATE_H_ 

#include <linux/stddef.h>
#include <linux/sys.h>

#include "syscall_hijack_public.h"
#include "rule_engine_public.h"

/* the tracker mutex. all operations involving filter_vars or meta data */
/* rule lists, etc) must take the lock.                                 */
extern struct semaphore tracker_sem;

/* we only support one mux function now, socketcall.               */
/* you need to update this section if you add more muxed syscalls. */
#define NR_MUX_SYSCALLS 1
#define MUX_NO_SYSCALL -1 /* no such muxed call */
#define MUX_SOCKETCALL 0  /* for socketcall */

/* translate a syscall_id for a mux syscall into an index to */
/* the mux_data array.                                       */
static inline int mux_data_index(int syscall_id)
{
	switch (syscall_id) {
		/* for socketcall */
	case (102): return MUX_SOCKETCALL;
	
        /* other mux syscalls would go here */
	
	/* for anything else */
	default: return MUX_NO_SYSCALL;
	}
}

struct mux_data_t {
	func_ptr mux_tracker;	    /* tracker for the syscall itself.      */
	int num_muxed;		    /* number of muxed functions.           */
	func_ptr* muxed_functions;  /* trackers for the muxed functions.    */
	func_ptr* demux_functions;  /* functions allowing re-calling the    */
	                            /* orig syscall from the function       */
	                            /* trackers.                            */
	sct_rule_list_t** before_rules; /* rules list, one per muxed function.  */
	sct_rule_list_t** after_rules;  /* rules list, one per muxed function.  */
};

extern struct mux_data_t mux_data[NR_MUX_SYSCALLS];

/* type of info stored for a tracked syscall. */
struct syscall_track_data {
	sct_rule_list_t* before_rules;  /* list of 'before' rules for syscall. */
	sct_rule_list_t* after_rules;   /* list of 'after' rules for syscall.  */
};

/* track data for all syscalls. */
extern struct syscall_track_data track_data[NR_syscalls];

/* lives in syscall_tracker_autogen.c */
extern func_ptr before_track_functions[];
extern func_ptr after_track_functions[];

/* we need this for some sct_membuf funkiness, to supply our own "copy to membuf" */ 
/* and "copy from membuf" functions. We can't use copy_to_user and copy_from_user */ 
/* directly, so we wrap them up in wrapper functions */ 
/* we can't use copy_to/from_user, as they're macros, so we wrap them */ 
extern unsigned long cp_to_user(void* to, const void *from, unsigned long n); 
extern unsigned long cp_from_user(void *to, const void *from, unsigned long n); 

#endif /* SCT_RULES_PRIVATE_H_ */
