/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/*
 * $Id: special_tracker.c,v 1.5 2002/06/17 23:30:34 mulix Exp $
 */

/* 
 * This file has the tracking functions for special system calls, such 
 * as exit(), execve() or fork(). 
 */ 

/* tracking function for syscall 'exit' (1). */
asmlinkage static long
track_before_exit(int error_code) {
	sct_action_t* action = NULL;
	int rule_id = -1;
	int dummy; 

	/* lock the semaphore */ 
	down(&tracker_sem);

	/* reset the vars */ 
	reset_all_vars();

	/* we've got one variable, the error code */ 
	set_var_indexed_callback(VT_PARAM, 0, global_param_callback, 
				 DT_INT, (void*)&error_code);

	/* did we match a rule? */ 
	action = sct_filter_syscall(track_data[1].before_rules,
				    1, &rule_id);

	if (action) {
		/* we did! perform the action */ 
		(void)sct_tracker_perform_action(action, rule_id, 1,
						 &dummy, "exit", "i" , 
						 error_code); 
	}
	/* release the semaphore */ 
	up(&tracker_sem);

	/* doesn't mean anything, silence gcc */ 
	return 0; 
}

/* tracking function for syscall 'fork' (2). */
asmlinkage static int
track_before_syscall_2(int* syscall_rc) {
	sct_syscall_retval_policy retval = SCT_USE_ORIG_RETVAL; 


	{
		sct_action_t* action = NULL;
		int rule_id = -1;
	
		down(&tracker_sem);
 		reset_all_vars();
		action = sct_filter_syscall(track_data[2].before_rules,
					2,
					&rule_id);
		if (action) {
			retval = sct_tracker_perform_action(
			action, rule_id, 2,
			syscall_rc, "fork",
			"v" ); 
		}
		up(&tracker_sem);
	}


	return retval;
}

/* tracking function for syscall 'fork' (2). */
asmlinkage static int
track_after_syscall_2(int syscall_rc) {
         int retval = 0; 



	{
		sct_action_t* action = NULL;
		int rule_id = -1;
		int dummy; 


		down(&tracker_sem);
		reset_all_vars();
		set_var_callback(VT_RETVAL, global_retval_callback, (void*)syscall_rc);
		action = sct_filter_syscall(track_data[2].after_rules,
					2,
					&rule_id);
		if (action) {
			sct_tracker_perform_action(
			action, rule_id, 2, &dummy, "fork",
			"v" );
		}
		up(&tracker_sem);
	}


	return retval; 
}

void init_special_syscalls(void)
{
	before_track_functions[1] = (func_ptr)track_before_exit; 
	before_track_functions[2] = (func_ptr)track_before_syscall_2; 
	after_track_functions[2] = (func_ptr)track_after_syscall_2; 
}

void cleanup_special_syscalls(void)
{
	; /* nothing */ 
}
