/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 guy keren, choo@actcom.co.il             */
/* License: GNU General Public License                              */

/*
 * $Id: string_util.c,v 1.4 2002/06/15 15:16:14 mulix Exp $
 */

#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <asm/uaccess.h>

#include "string_util.h"

/* str is in uspace, page is PAGE_SIZE */ 
static int do_getstring(const char* str, char* page)
{
	int retval;
	unsigned long len = strnlen_user((char*)str, PAGE_SIZE); 
	if (!len)
		return -EFAULT; 
	
	if (len > PAGE_SIZE)
		len = PAGE_SIZE; 

	/* returns the ammount of data NOT copied */ 
	retval = copy_from_user(page, str, len);
	if (retval > 0)
		return -EFAULT; 

	return retval;
}


int getstring(const char* str, char** kern_str)
{
	int result;
	char *tmp;
	
	result = -ENOMEM;
	tmp = (char *)kmalloc(PAGE_SIZE, GFP_KERNEL); 
	if (!tmp)
		return -ENOMEM; 

	if ((result = do_getstring(str, tmp)) < 0) {
		kfree(tmp);
		*kern_str = NULL;
	} else {
		*kern_str = tmp;
	}
	return result; 
}
