/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 guy keren, choo@actcom.co.il             */
/* License: GNU General Public License                              */

/* 
 * $Id: struct_handling.h,v 1.6 2002/06/15 15:16:14 mulix Exp $
 */ 

#ifndef STRUCT_HANDLING_H_
#define STRUCT_HANDLING_H_

/* function used to log the contents of a struct. */
typedef int (*struct_logger_t)(void* void_s, char* buf, int buflen);

/* structure used to store info about type data for a struct type. */
struct struct_data {
    long* offsets_vec;          /* offsets of struct's fields.     */
    sct_data_type_t* types_vec; /* types of struct's fields.       */
    long num_fields;            /* number of fields in struct.     */
    int struct_size;		/* size of the struct.             */
    struct_logger_t logger;	/* struct logger function.         */
};

/* structure used to pass data to a struct logger function. */
struct struct_logger_data {
    unsigned int struct_type;
    int param_index;
};

/* forward reference */
extern struct struct_data struct_type_data[];
extern int struct_type_data_sz; /* size of the array */


/* get the struct data for the struct of the given type. */
static inline struct struct_data*
get_struct_data(unsigned int struct_type)
{
	if (struct_type >= struct_type_data_sz)
		BUG(); /* can't use BUG_ON(), backward compat. */
	
	return &struct_type_data[struct_type];
}

#endif /* STRUCT_HANDLING_H_ */

